const CACHE_NAME = "cyberpunk-cache-v1";
const OFFLINE_URL = "/offline.html"; // Diese Seite wird angezeigt, wenn offline
const BASE_PATH = "/cpred/";
const SITEMAP_URL = BASE_PATH + "sitemap.json";

const urlsToCache = [
  BASE_PATH,
  BASE_PATH + "offline.html",
  /* CSS-Dateien */
  BASE_PATH + "assets/css/main.css",
  BASE_PATH + "assets/css/normalize.css",
  BASE_PATH + "assets/css/cyberpunk.css",
  /* JS-Dateien */
  BASE_PATH + "assets/js/materialize.min.js",
  BASE_PATH + "assets/js/service-worker.js",
  /* PWA-Dateien */
  BASE_PATH + "assets/manifest.json",
  BASE_PATH + "assets/img/icon-192.png",
  BASE_PATH + "assets/img/icon-512.png",
  SITEMAP_URL,
  /* Bilder, Schriften, etc. */
  BASE_PATH + "assets/fonts/BlenderProBook.woff2",
  BASE_PATH + "assets/fonts/Cyberpunk.otf",
  BASE_PATH + "assets/fonts/Oxanium.woff2",
  BASE_PATH + "assets/img/Netrunner.png",
  BASE_PATH + "assets/img/Tech.png",
  /* Game Data */
  BASE_PATH + "assets/game-data/housings.json",
  BASE_PATH + "assets/game-data/lifestyles.json",
  BASE_PATH + "assets/game-data/skills.json",
  BASE_PATH + "assets/game-data/characters/chase.json",
  BASE_PATH + "assets/game-data/characters/carver.json",
];

/**
 * Install-Event: Cache Standarddateien
 */
self.addEventListener("install", (event) => {
  event.waitUntil(
    caches.open(CACHE_NAME).then((cache) => {
      return cache.addAll(urlsToCache);
    })
  );
});

/**
 * Fetch-Event: Cache-First Strategie mit Offline-Fallback
 */
self.addEventListener("fetch", (event) => {
  event.respondWith(
    caches.match(event.request).then((response) => {
      return response || fetch(event.request).catch(() => {
        return caches.match(OFFLINE_URL);
      });
    })
  );
});

/**
 * Wenn die Startseite geladen wird, lade automatisch alle Seiten aus der Sitemap in den Cache
 */
self.addEventListener("sync", (event) => {
  if (event.tag === "cache-all-pages") {
    event.waitUntil(
      fetch(SITEMAP_URL)
        .then((response) => response.json())
        .then((data) => {
          return caches.open(CACHE_NAME).then((cache) => {
            return Promise.all(
              data.pages.map((page) => {
                return fetch(page.url)
                  .then((response) => cache.put(page.url, response))
                  .catch((err) => console.error("Fehler beim Cachen:", err));
              })
            );
          });
        })
    );
  }
});

/**
 * Aktivierungs-Event: Löscht alten Cache
 */
self.addEventListener("activate", (event) => {
  event.waitUntil(
    caches.keys().then((cacheNames) => {
      return Promise.all(
        cacheNames.map((cache) => {
          if (cache !== CACHE_NAME) {
            return caches.delete(cache);
          }
        })
      );
    })
  );
});
